@@ -22,18 +22,15 @@ def add_cursor_to_mark(unit_chart,cursor_type):
     
     return unit_chart
 
-def recursively_add_to_mark(chart,cursor_type):
-    if hasattr(chart,'mark') :
-        chart = add_cursor_to_mark(chart,cursor_type)
 
-# todo fix this, to not add click pointer to background 
+
+
     
-    if  cursor_type != "pointer" and (hasattr(chart,'mark') or hasattr(chart,'layer')):
-        chart.view = {
-            "cursor":cursor_type,
-            "stroke":None
-        }
-        return chart
+
+def recursively_add_to_mark(chart,functions_to_call):
+    for function in functions_to_call:
+       chart = function(chart)
+
 
     attributes_for_recursion = ['layer','hconcat','vconcat']
     for attribute in attributes_for_recursion:
@@ -41,7 +38,7 @@ def recursively_add_to_mark(chart,cursor_type):
         # instead, it should see if any exists, and if it does, it should use that as the item to search
         if alt_get(chart,attribute):
           for unit_spec in chart[attribute]:
-              unit_spec = recursively_add_to_mark(unit_spec,cursor_type)
+              unit_spec = recursively_add_to_mark(unit_spec,functions_to_call)
  
     return chart
 
@@ -81,8 +78,20 @@ def create_selection(chart,interaction):
     # check if any axis is aggregate
     
     if interaction.action['trigger'] == "drag":
-        encodings =  ['x','y'] # by default
-        encodings = [encoding for encoding in encodings if not check_axis_aggregate(chart,encoding)]
+        x_is_aggregate = check_axis_aggregate(chart,'x') 
+        y_is_aggregate = check_axis_aggregate(chart,'y')
+            
+        x_field = get_field_from_encoding(chart,'x')
+        y_field = get_field_from_encoding(chart,'y')
+
+        x_is_meaningful = x_field and 'level' not in x_field and not x_is_aggregate
+        y_is_meaningful = y_field and 'level' not in y_field and not y_is_aggregate
+
+        encodings =  [] # by default
+        if x_is_meaningful :
+            encodings.append('x')
+        if y_is_meaningful:
+            encodings.append('y')
 
         # if it is a line chart without additional encodings options, use x
         has_options = getattr(interaction,'options',None) != None
@@ -103,21 +112,29 @@ def create_selection(chart,interaction):
             field = get_field_from_encoding(chart,interaction.action['target'])
             selection=alt.selection_point(name=name, encodings=[interaction.action['target']], fields=[field])
         else: 
-            x_is_aggregate = check_axis_aggregate(chart,'x')
+            x_is_aggregate = check_axis_aggregate(chart,'x') 
             y_is_aggregate = check_axis_aggregate(chart,'y')
 
             fields = []
             if get_field_from_encoding(chart,'column'):
                 fields = [get_field_from_encoding(chart,'column')]
             
+            x_field = get_field_from_encoding(chart,'x')
+            y_field = get_field_from_encoding(chart,'y')
 
-            if  x_is_aggregate and not y_is_aggregate:
+            x_is_meaningful = x_field and 'level' not in x_field and not x_is_aggregate
+            y_is_meaningful = y_field and 'level' not in y_field and not y_is_aggregate
+            
+            if  x_is_meaningful and not y_is_meaningful:
+                fields.append(x_field)
                 # if x is aggregated (ie is a count), then add y field to selection 
                 selection=alt.selection_point(name=name, encodings=['y'],fields=fields)
-            elif not  x_is_aggregate and  y_is_aggregate:
+            elif not  x_is_meaningful and  y_is_meaningful:
                 # if both of them are 
+                fields.append(y_field)
+
                 selection=alt.selection_point(name=name, encodings=['x'],fields=fields)
-            elif not x_is_aggregate and not y_is_aggregate:
+            elif not x_is_meaningful and not x_is_meaningful:
                 selection=alt.selection_point(name=name, encodings=['x','y'],fields=fields)
 
     if interaction.action['trigger'] == "type":
@@ -134,9 +151,21 @@ def create_selection(chart,interaction):
         name = ALX_SELECTION_PREFIX+'panzoom'+ALX_SELECTION_SUFFIX[interaction.effect['transform']]
         selection = alt.selection_interval(name=name,bind="scales", encodings=encodings)
     
+    if interaction.action['trigger'] == "mouseover":
+        params = {
+            "on": "mouseover",
+            "empty": False,
+            "clear": "mouseout",
+        }
+
+        if 'nearest' in interaction.action:
+            params['nearest'] = True
+
+        selection = alt.selection_point(**params)
+
     return selection 
 
-def add_colors(chart,data,field):
+def add_colors(chart,data,field,legend=alt.Legend()):
     unique_values = pd.unique(data)
     unique_values.sort()
     domain = ['Group']
@@ -148,34 +177,62 @@ def add_colors(chart,data,field):
         else: 
             range.append("lightgray")
     color_scale = alt.Scale(domain=domain,range=range)
-    chart=chart.encode(alt.Color(field=field,scale=color_scale))
+    chart=chart.encode(alt.Color(field=field,scale=color_scale,legend=legend))
     return chart
 
 def apply_effect(chart,interaction,selection):
     attributes_for_recursion = ['layer','hconcat','vconcat']
-    for attribute in attributes_for_recursion:
 
+
+    for attribute in attributes_for_recursion:
         if alt_get(chart,attribute):
+
             specs = []
             for unit_spec in chart[attribute]:
-                #if alt_get(chart,'data') and not is_undefined(chart.data):
-                #    print('adding data',len(chart.data))
-                    #unit_spec.data = chart.data
-
-                
                 specs.append(apply_effect(unit_spec,interaction,selection))
             chart[attribute] = specs
 
-
+    
+    
     if getattr(chart,'mark',None) is not None:
         chart =  apply_effect_recurse(chart,interaction,selection)
-    
+    elif alt_get(chart,'spec') :
+
+        chart = apply_effect_recurse(chart,interaction,selection)
     return chart
 
+def add_tooltip_chart(chart,interaction,selection):
+    if interaction.action['trigger'] == "mouseover":
+        
+        def add_tooltip_to_mark(chart):
+            tooltip = {"content": "data"}
+            
+            if isinstance(chart.mark,str):
+                mark_type = chart.mark
+                chart.mark = alt.MarkDef(type=mark_type,tooltip=tooltip)
+            else: 
+                chart.mark.tooltip = tooltip
+            
+            return chart
+
+        recursively_add_to_mark(chart,[add_tooltip_to_mark])
+    return chart
+
+
 def apply_effect_recurse(previous_chart,interaction,selection):
     chart = previous_chart.copy(deep=True)
     # alter the chart object to allow for interaction
     # apply the transform 
+    # if chart is a facet, apply effect to the spec and then return the chart
+    if alt_get(chart,'spec'):
+
+        copied_chart = chart.spec.copy(deep=True)
+  
+        copied_chart.data = chart.data
+        
+        chart.spec = apply_effect_recurse(copied_chart,interaction,selection)
+        return chart
+
     
     if interaction.effect['transform'] == "filter":
          chart = filter_chart(chart,interaction,selection)
@@ -184,13 +241,15 @@ def apply_effect_recurse(previous_chart,interaction,selection):
     
     if interaction.effect['transform'] == "highlight":
         chart = highlight_chart(chart,interaction,selection)
-    
     if interaction.effect['transform'] == "group":
         chart = group_chart(chart,interaction,selection)
         
     if interaction.effect['transform'] == "scale_bind":
         chart = pan_zoom_chart(chart,interaction,selection)
 
+    if interaction.effect['transform'] ==  "tooltip":
+        chart = add_tooltip_chart(chart,interaction,selection)
+        
     return chart
 
 
@@ -262,7 +321,7 @@ def group_chart(chart,interaction,selection):
             if_2 = f'''
             if(isDefined({selection.name}["{group_name}"]),
               datum["{group_name}"] == "Group"?
-                  {selection.name}["{group_name}"] : datum["{group_name}"],
+                  "Group: ["+{selection.name}["{group_name}"] +"]": datum["{group_name}"],
               datum["{group_name}"]
             )
             '''
@@ -280,7 +339,7 @@ def group_chart(chart,interaction,selection):
             if check_if_line(chart):
                 # do 
                 if category == 'color':
-                    chart = add_colors(chart,chart.data[field],group_name)
+                    chart = add_colors(chart,chart.data[field],group_name,legend=None)
 
                 # add mouse overlay
                 params = chart.params
@@ -290,6 +349,8 @@ def group_chart(chart,interaction,selection):
                 if isinstance(properties,str):
                   interaction_overlay=interaction_overlay.mark_line(strokeWidth= 8, stroke="transparent")
                 else:
+                  # TODO: fix this, breaking double line chart overview+detail
+                  properties = {}
                   properties['strokeWidth'] = 8
                   properties['stroke'] = "transparent"
                   interaction_overlay=interaction_overlay.mark_line(**properties)
@@ -330,12 +391,15 @@ def filter_chart(chart,interaction,selection):
     selection_type = getattr(selection.param,'select',{})
     encodings = getattr(selection_type,'encodings',None) or ["x","y"] # default to x and y to maintain reference of chart
     
+    # if the interaction occurs via a widget, then axis should be altered
+    interaction_occurs_on_chart = not interaction.action['trigger'] == "type"
+
     # fix the encoding scales so that the view remains static 
-    if encodings and not is_undefined(encodings): 
+    if encodings and not is_undefined(encodings) and interaction_occurs_on_chart: 
         for encoding in encodings:
             field = get_field_from_encoding(chart,encoding)
             
-            if not is_undefined(chart.data) and field:
+            if not is_undefined(chart.data) and field and field in chart.data.columns:
                 extent = extent_from_column(chart.data,field)
                 # TODO: copy the existing scale, just overwrite the domain
                 scale = alt.Scale(domain=extent)
@@ -347,15 +411,19 @@ def pan_zoom_chart(chart,interaction,selection):
     return chart.add_params(selection)
 
 def highlight_chart(chart,interaction,selection):
-     # for text box interaction, use query filter
-   
+    # for text box interaction, use query filter
 
     # if any of the axes are aggregated
+    x_agg = check_axis_aggregate(chart,'x')
+    y_agg = check_axis_aggregate(chart,'y')
+
     x_binned = check_axis_binned(chart,'x')
     y_binned = check_axis_binned(chart,'y')
+
     is_line = check_if_line(chart)
 
     if  is_line:
+
         # for line charts, create a new layer with a color scale that maps to light gray
         color = get_field_from_encoding(chart,'color')      
 
@@ -388,11 +456,9 @@ def highlight_chart(chart,interaction,selection):
             chart.layer[1].transform.insert(0,filter_transform)
         else:
             chart.layer[1].transform = [filter_transform]
-
-    elif not x_binned and not y_binned :
+    elif (not x_agg and not y_agg) and (not x_binned and not y_binned) :
         # non-binned charts ()
 
-
         # if the chart already has a color encoding, use that as a conditional
         highlight = get_field_from_encoding(chart,'color') or alt.value('steelblue')
 
@@ -407,9 +473,11 @@ def highlight_chart(chart,interaction,selection):
         chart = add_encoding(chart,color)
         
     else:
+
         # used for any elements where height, width, etc are controlled by filter 
         color_encoding = chart.encoding.color
         #chart.encoding.color.scale=alt.Scale(scheme='greys')
+
         chart.encoding.color = alt.value('lightgray')
         chart = chart + chart 
         chart.layer[1].encoding.color = color_encoding
@@ -422,6 +490,7 @@ def highlight_chart(chart,interaction,selection):
             chart.layer[1].transform.insert(0,filter_transform)
         else:
             chart.layer[1].transform = [filter_transform]
+
     return chart 
 
 
@@ -458,10 +527,12 @@ point = {"trigger":"click"}
 color = {"trigger":"click","target":"color"}
 text = {"trigger":"type"}
 brush = {"trigger":"drag"}
-hover = {"trigger":"mouseover"}
+#hover = {"trigger":"mouseover"}
 
-def tooltip_hover():
-    return Interaction(effect=tooltip,action=hover)
+def tooltip_hover(nearest=False):
+    action = {"trigger":"mouseover"}
+    action['nearest'] = nearest
+    return Interaction(effect=tooltip,action=action)
 
 def highlight_brush(options=None):
     return Interaction(effect=highlight,action=brush,options=options)
@@ -506,6 +577,18 @@ def process_effects(chart,effects):
       chart = process_highlights(chart,effects['highlight'])
     elif 'group' in effects:
       chart = process_groups(chart,effects['group'])
+    elif 'tooltip' in effects:
+      chart = process_tooltip(chart,effects['tooltip'])
+
+    return chart
+
+def process_tooltip(chart,tooltip):
+    if isinstance(tooltip, Interaction):
+        parameter = tooltip.get_selection()
+        if parameter is None:
+            parameter = create_selection(chart,tooltip)
+        chart = apply_effect(chart,tooltip,parameter)
+    
     return chart
 
 def process_highlights(chart,highlights):
@@ -517,7 +600,6 @@ def process_highlights(chart,highlights):
           parameter = highlight.get_selection()
           if parameter is None:
               parameter = create_selection(chart,highlight)
-
           chart = apply_effect(chart,highlight,parameter)
   return chart
 
@@ -549,23 +631,56 @@ def process_groups(chart,groups):
   return chart
 def add_cursor(chart,interaction):
     if interaction.action['trigger'] == "drag":
-        chart = recursively_add_to_mark(chart,'crosshair')
-
+        def add_crosshair(chart):
+            if hasattr(chart,'mark') :
+                return add_cursor_to_mark(chart,'crosshair')
+            return chart
+       
+        chart = recursively_add_to_mark(chart,[add_crosshair])
+        if  (hasattr(chart,'mark') or hasattr(chart,'layer')):
+            chart.view = {
+                "cursor":'crosshair',
+                "stroke":None
+            }
         #chart.view ={"cursor":"crosshair","stroke":None}
     if interaction.action['trigger'] == "click":
-        chart = recursively_add_to_mark(chart,'pointer')
+        def add_pointer(chart):
+            if hasattr(chart,'mark') :
+
+                return add_cursor_to_mark(chart,'pointer')
+            return chart
+        
+        chart = recursively_add_to_mark(chart,[add_pointer])
     if interaction.action['trigger'] == "panzoom":
-        chart = recursively_add_to_mark(chart,'move')
-        #chart.view ={"cursor":"move"}
+        def add_move(chart):
+            if hasattr(chart,'mark') :
+                return add_cursor_to_mark(chart,'move')
+            return chart
+        
+        def add_move_to_view(chart):
+            if  (hasattr(chart,'mark') or hasattr(chart,'layer')):
+                chart.view = {
+                    "cursor":'move',
+                    "stroke":None
+                }
+            return chart
+        chart = recursively_add_to_mark(chart,[add_move,add_move_to_view])
     return chart
 
 def add_interaction(chart, interaction):
     
     parameter = create_selection(chart,interaction)
     interaction.set_selection(parameter)
-    chart=chart.add_params(parameter)
+    # spec charts need param added to the spec itself, otherwise you get duplicate signals
+    if alt_get(chart,'spec'):
+        spec = chart.spec
+        spec = spec.add_params(parameter)
+        chart.spec = spec
+    else:
+        chart=chart.add_params(parameter)
     chart =  apply_effect(chart,interaction,parameter)
     chart = add_cursor(chart,interaction)
+   
 
     return chart
 
